@@ -2,12 +2,13 @@
 #'
 #' A simple delay-difference assessment model using a
 #' time-series of catches and a relative abundance index and coded in TMB. The model
-#' is conditioned on effort and estimates predicted catch. In the state-space version,
-#' recruitment deviations from the stock-recruit relationship are estimated.
+#' can be conditioned on either (1) effort and estimates predicted catch or (2) catch and estimates a predicted index.
+#' In the state-space version, recruitment deviations from the stock-recruit relationship are estimated.
 #'
 #' @param x An index for the objects in \code{Data} when running in closed loop simulation.
 #' Otherwise, equals to 1 when running an assessment.
 #' @param Data An object of class \linkS4class{Data}.
+#' @param condition A string to indicate whether to condition the model on catch or effort (ratio of catch and index).
 #' @param SR Stock-recruit function (either \code{"BH"} for Beverton-Holt or \code{"Ricker"}).
 #' @param rescale A multiplicative factor that rescales the catch in the assessment model, which
 #' can improve convergence. By default, \code{"mean1"} scales the catch so that time series mean is 1, otherwise a numeric.
@@ -17,8 +18,9 @@
 #' @param fix_U_equilibrium Logical, whether the equilibrium harvest rate prior to the first year of the model is
 #' estimated. If TRUE, U_equilibruim is fixed to value provided in start (if provided), otherwise, equal to zero
 #' (assumes virgin conditions).
-#' @param fix_omega Logical, whether the standard deviation of the catch is fixed. If \code{TRUE},
-#' omega is fixed to value provided in \code{start} (if provided), otherwise, value based on \code{Data@@CV_Cat}.
+#' @param fix_sd Logical, whether the standard deviation of the data in the likelihood (index for conditioning on catch or
+#' catch for conditioning on effort). If \code{TRUE}, the SD is fixed to value provided in \code{start} (if provided), otherwise,
+#'  value based on either \code{Data@@CV_Cat} or \code{Data@@CV_Ind}.
 #' @param fix_tau Logical, the standard deviation of the recruitment deviations is fixed. If \code{TRUE},
 #' tau is fixed to value provided in \code{start} (if provided), otherwise, equal to 1.
 #' @param integrate Logical, whether the likelihood of the model integrates over the likelihood
@@ -36,8 +38,7 @@
 #' @param inner.control A named list of arguments for optimization of the random effects, which
 #' is passed on to \code{\link[TMB]{newton}} via \code{\link[TMB]{MakeADFun}}.
 #' @param ... Additional arguments (not currently used).
-#' @return An object of \code{\linkS4class{Assessment}} containing objects and output
-#' from TMB.
+#' @return An object of \code{\linkS4class{Assessment}} containing objects and output from TMB.
 #' @details
 #' To provide starting values for \code{DD_TMB}, a named list can be provided for \code{R0} (virgin recruitment),
 #' \code{h} (steepness), and \code{q} (catchability coefficient) via the \code{start} argument (see example).
@@ -87,13 +88,44 @@
 #' @seealso \link{plot.Assessment} \link{summary.Assessment} \link{retrospective} \link{profile} \link{make_MP}
 #' @useDynLib MSEtool
 #' @export
-DD_TMB <- function(x = 1, Data, SR = c("BH", "Ricker"), rescale = "mean1", start = NULL, fix_h = TRUE,
+DD_TMB <- function(x = 1, Data, condition = c("catch", "effort"), SR = c("BH", "Ricker"), rescale = "mean1", start = NULL, fix_h = TRUE,
                    fix_U_equilibrium = TRUE, silent = TRUE, opt_hess = FALSE, n_restart = ifelse(opt_hess, 0, 1),
                    control = list(iter.max = 5e3, eval.max = 1e4), ...) {
+  condition <- match.arg(condition)
+  DD_(x = x, Data = Data, state_space = FALSE, condition = condition, SR = SR, rescale = rescale, start = start,
+      fix_h = fix_h, fix_U_equilibrium = fix_U_equilibrium,
+      fix_omega = condition == "catch", fix_sigma = condition == "effort",
+      fix_tau = TRUE, integrate = FALSE, silent = silent, opt_hess = opt_hess, n_restart = n_restart,
+      control = control, inner.control = list(), ...)
+}
+class(DD_TMB) <- "Assess"
+
+
+#' @rdname DD_TMB
+#' @useDynLib MSEtool
+#' @export
+DD_SS <- function(x = 1, Data, condition = c("catch", "effort"), SR = c("BH", "Ricker"), rescale = "mean1", start = NULL,
+                  fix_h = TRUE, fix_U_equilibrium = TRUE, fix_sd = FALSE, fix_tau = TRUE,
+                  integrate = FALSE, silent = TRUE, opt_hess = FALSE, n_restart = ifelse(opt_hess, 0, 1),
+                  control = list(iter.max = 5e3, eval.max = 1e4), inner.control = list(), ...) {
+  condition <- match.arg(condition)
+  DD_(x = x, Data = Data, state_space = TRUE, condition = condition, SR = SR, rescale = rescale, start = start,
+      fix_h = fix_h, fix_U_equilibrium = fix_U_equilibrium,
+      fix_omega = condition == "catch" | fix_sd, fix_sigma = condition == "effort" | fix_sd,
+      fix_tau = fix_tau, integrate = integrate, silent = silent, opt_hess = opt_hess, n_restart = n_restart,
+      control = control, inner.control = inner.control, ...)
+}
+class(DD_SS) <- "Assess"
+
+DD_ <- function(x = 1, Data, state_space = FALSE, condition = c("catch", "effort"), SR = c("BH", "Ricker"), rescale = "mean1", start = NULL,
+                fix_h = TRUE, fix_U_equilibrium = TRUE, fix_omega = FALSE, fix_sigma = FALSE, fix_tau = TRUE,
+                integrate = FALSE, silent = TRUE, opt_hess = FALSE, n_restart = ifelse(opt_hess, 0, 1),
+                control = list(iter.max = 5e3, eval.max = 1e4), inner.control = list(), ...) {
   dependencies <- "Data@Cat, Data@Ind, Data@Mort, Data@L50, Data@vbK, Data@vbLinf, Data@vbt0, Data@wla, Data@wlb, Data@MaxAge"
   dots <- list(...)
   start <- lapply(start, eval, envir = environment())
 
+  condition <- match.arg(condition)
   SR <- match.arg(SR)
   Winf = Data@wla[x] * Data@vbLinf[x]^Data@wlb[x]
   age <- 1:Data@MaxAge
@@ -110,10 +142,16 @@ DD_TMB <- function(x = 1, Data, SR = c("BH", "Ricker"), rescale = "mean1", start
   Year <- Data@Year[yind]
   C_hist <- Data@Cat[x, yind]
   I_hist <- Data@Ind[x, yind]
-  E_hist <- C_hist/I_hist
-  if(any(is.na(E_hist))) stop("Missing values in catch and index in Data object.")
-  E_rescale <- 1/mean(E_hist)
-  E_hist <- E_hist * E_rescale
+
+  if(condition == "effort") {
+    E_hist <- C_hist/I_hist
+    if(any(is.na(E_hist))) stop("Missing values in catch and index in Data object.")
+    E_rescale <- 1/mean(E_hist)
+    E_hist <- E_hist * E_rescale
+  } else {
+    E_hist <- rep(1, length(yind))
+  }
+
   ny <- length(C_hist)
   k <- ceiling(a50V)  # get age nearest to 50% vulnerability (ascending limb)
   k[k > Data@MaxAge/2] <- ceiling(Data@MaxAge/2)  # to stop stupidly high estimates of age at 50% vulnerability
@@ -124,7 +162,8 @@ DD_TMB <- function(x = 1, Data, SR = c("BH", "Ricker"), rescale = "mean1", start
 
   if(rescale == "mean1") rescale <- 1/mean(C_hist)
   data <- list(model = "DD", S0 = S0, Alpha = Alpha, Rho = Rho, ny = ny, k = k,
-               wk = wk, E_hist = E_hist, C_hist = C_hist * rescale, SR_type = SR)
+               wk = wk, C_hist = C_hist, rescale = rescale, I_hist = I_hist, E_hist = E_hist, SR_type = SR,
+               condition = condition, state_space = as.integer(state_space))
   LH <- list(LAA = la, WAA = wa, maxage = Data@MaxAge, A50 = k)
 
   params <- list()
@@ -138,146 +177,10 @@ DD_TMB <- function(x = 1, Data, SR = c("BH", "Ricker"), rescale = "mean1", start
         params$transformed_h <- log(start$h[1] - 0.2)
       }
     }
-    if(!is.null(start$q) && is.numeric(start$q)) params$log_q <- log(start$q[1])
-    if(!is.null(start$U_equilibrium) && is.numeric(start$U_equilibrium)) params$U_equilibrium <- start$U_equililbrium
-  }
-  if(is.null(params$log_R0)) {
-    params$log_R0 <- ifelse(is.null(Data@OM$R0[x]), log(4 * mean(data$C_hist)), log(1.5 * rescale * Data@OM$R0[x]))
-  }
-  if(is.null(params$transformed_h)) {
-    h_start <- ifelse(is.na(Data@steep[x]), 0.9, Data@steep[x])
-    if(SR == "BH") {
-      h_start <- (h_start - 0.2)/0.8
-      params$transformed_h <- logit(h_start)
-    } else {
-      params$transformed_h <- log(h_start - 0.2)
-    }
-  }
-  if(is.null(params$log_q)) params$log_q <- log(1)
-  if(is.null(params$U_equilibrium)) params$U_equilibrium <- 0
-
-  info <- list(Year = Year, data = data, params = params, I_hist = I_hist, LH = LH, rescale = rescale,
-               E_rescale = E_rescale, control = control)
-
-  map <- list()
-  if(fix_h) map$transformed_h <- factor(NA)
-  if(fix_U_equilibrium) map$U_equilibrium <- factor(NA)
-
-  obj <- MakeADFun(data = info$data, parameters = info$params, hessian = TRUE, map = map, DLL = "MSEtool", silent = silent)
-
-  mod <- optimize_TMB_model(obj, control, opt_hess, n_restart)
-  opt <- mod[[1]]
-  SD <- mod[[2]]
-  report <- obj$report(obj$env$last.par.best)
-
-  if(rescale != 1) {
-    vars_div <- c("B0", "B", "Cpred", "N0", "N", "R0", "R")
-    vars_mult <- c("Brec")
-    var_trans <- c("R0")
-    fun_trans <- c("/")
-    fun_fixed <- c("log")
-    rescale_report(vars_div, vars_mult, var_trans, fun_trans, fun_fixed)
-  }
-  Yearplusone <- c(Year, max(Year) + 1)
-  Yearplusk <- c(Year, max(Year) + 1:k)
-
-  nll_report <- ifelse(is.character(opt), report$nll, opt$objective)
-  Assessment <- new("Assessment", Model = "DD_TMB", Name = Data@Name, conv = !is.character(SD) && SD$pdHess,
-                    B0 = report$B0, R0 = report$R0, N0 = report$N0,
-                    SSB0 = report$B0, VB0 = report$B0, h = report$h,
-                    U = structure(report$U, names = Year),
-                    B = structure(report$B, names = Yearplusone),
-                    B_B0 = structure(report$B/report$B0, names = Yearplusone),
-                    SSB = structure(report$B, names = Yearplusone),
-                    SSB_SSB0 = structure(report$B/report$B0, names = Yearplusone),
-                    VB = structure(report$B, names = Yearplusone),
-                    VB_VB0 = structure(report$B/report$B0, names = Yearplusone),
-                    R = structure(report$R, names = Yearplusk),
-                    N = structure(report$N, names = Yearplusone),
-                    Obs_Catch = structure(C_hist, names = Year),
-                    Obs_Index = structure(I_hist, names = Year),
-                    Catch = structure(report$Cpred, names = Year),
-                    Index = structure(report$Cpred/(C_hist/I_hist), names = Year),
-                    NLL = structure(c(nll_report, report$nll - report$penalty, report$penalty),
-                                    names = c("Total", "Catch", "Penalty")),
-                    info = info, obj = obj, opt = opt,
-                    SD = SD, TMB_report = report, dependencies = dependencies)
-
-  if(Assessment@conv) {
-    ref_pt <- get_MSY_DD(info$data, report$Arec, report$Brec)
-    report <- c(report, ref_pt)
-
-    Assessment@UMSY <- report$UMSY
-    Assessment@MSY <- report$MSY
-    Assessment@BMSY <- Assessment@SSBMSY <- Assessment@VBMSY <- report$BMSY
-    Assessment@U_UMSY <- structure(report$U/report$UMSY, names = Year)
-    Assessment@B_BMSY <- Assessment@SSB_SSBMSY <- Assessment@VB_VBMSY <- structure(report$B/report$BMSY, names = Yearplusone)
-    Assessment@TMB_report <- report
-  }
-  return(Assessment)
-}
-class(DD_TMB) <- "Assess"
-
-
-#' @describeIn DD_TMB State-Space version of Delay-Difference model
-#' @useDynLib MSEtool
-#' @export
-DD_SS <- function(x = 1, Data, SR = c("BH", "Ricker"), rescale = "mean1", start = NULL,
-                  fix_h = TRUE, fix_U_equilibrium = TRUE, fix_omega = FALSE, fix_tau = TRUE,
-                  integrate = FALSE, silent = TRUE, opt_hess = FALSE, n_restart = ifelse(opt_hess, 0, 1),
-                  control = list(iter.max = 5e3, eval.max = 1e4), inner.control = list(), ...) {
-  dependencies <- "Data@Cat, Data@Ind, Data@Mort, Data@L50, Data@vbK, Data@vbLinf, Data@vbt0, Data@wla, Data@wlb, Data@MaxAge"
-  dots <- list(...)
-  start <- lapply(start, eval, envir = environment())
-  if("fix_sigma" %in% names(dots)) fix_omega <- dots$fix_sigma # For backwards compatibility
-
-  SR <- match.arg(SR)
-  Winf = Data@wla[x] * Data@vbLinf[x]^Data@wlb[x]
-  age <- 1:Data@MaxAge
-  la <- Data@vbLinf[x] * (1 - exp(-Data@vbK[x] * ((age - Data@vbt0[x]))))
-  wa <- Data@wla[x] * la^Data@wlb[x]
-  a50V <- iVB(Data@vbt0[x], Data@vbK[x], Data@vbLinf[x],  Data@L50[x])
-  a50V <- max(a50V, 1)
-  if(any(names(dots) == "yind")) {
-    yind <- eval(dots$yind)
-  } else {
-    ystart <- which(!is.na(Data@Cat[x, ] + Data@Ind[x, ]))[1]
-    yind <- ystart:length(Data@Cat[x, ])
-  }
-  Year <- Data@Year[yind]
-  C_hist <- Data@Cat[x, yind]
-  I_hist <- Data@Ind[x, yind]
-  E_hist <- C_hist/I_hist
-  if(any(is.na(E_hist))) stop("Missing values in catch and index in Data object.")
-  E_rescale <- 1/mean(E_hist)
-  E_hist <- E_hist * E_rescale
-  ny <- length(C_hist)
-  k <- ceiling(a50V)  # get age nearest to 50% vulnerability (ascending limb)
-  k[k > Data@MaxAge/2] <- ceiling(Data@MaxAge/2)  # to stop stupidly high estimates of age at 50% vulnerability
-  Rho <- (wa[k + 2] - Winf)/(wa[k + 1] - Winf)
-  Alpha <- Winf * (1 - Rho)
-  S0 <- exp(-Data@Mort[x])  # get So survival rate
-  wk <- wa[k]
-
-  if(rescale == "mean1") rescale <- 1/mean(C_hist)
-  data <- list(model = "DD_SS", S0 = S0, Alpha = Alpha, Rho = Rho, ny = ny, k = k,
-               wk = wk, E_hist = E_hist, C_hist = C_hist * rescale, SR_type = SR)
-  LH <- list(LAA = la, WAA = wa, maxage = Data@MaxAge, A50 = k)
-
-  params <- list()
-  if(!is.null(start)) {
-    if(!is.null(start$R0) && is.numeric(start$R0)) params$log_R0 <- log(start$R0[1] * rescale)
-    if(!is.null(start$h) && is.numeric(start$h)) {
-      if(SR == "BH") {
-        h_start <- (start$h[1] - 0.2)/0.8
-        params$transformed_h <- logit(h_start)
-      } else {
-        params$transformed_h <- log(start$h[1] - 0.2)
-      }
-    }
-    if(!is.null(start$q) && is.numeric(start$q)) params$log_q <- log(start$q[1])
+    if(!is.null(start$q_effort) && is.numeric(start$q_effort)) params$log_q_effort <- log(start$q_effort[1])
     if(!is.null(start$U_equilibrium) && is.numeric(start$U_equilibrium)) params$U_equilibrium <- start$U_equililbrium
     if(!is.null(start$omega) && is.numeric(start$omega)) params$log_omega <- log(start$omega[1])
+    if(!is.null(start$sigma) && is.numeric(start$sigma)) params$log_sigma <- log(start$sigma[1])
     if(!is.null(start$tau) && is.numeric(start$tau)) params$log_tau <- log(start$tau[1])
   }
   if(is.null(params$log_R0)) {
@@ -292,26 +195,31 @@ DD_SS <- function(x = 1, Data, SR = c("BH", "Ricker"), rescale = "mean1", start
       params$transformed_h <- log(h_start - 0.2)
     }
   }
-  if(is.null(params$log_q)) params$log_q <- log(1)
+  if(is.null(params$log_q_effort)) params$log_q_effort <- log(1)
   if(is.null(params$U_equilibrium)) params$U_equilibrium <- 0
   if(is.null(params$log_omega)) {
-    sigmaC <- max(0.05, sdconv(1, Data@CV_Cat[x]))
-    params$log_omega <- log(sigmaC)
+    params$log_omega <- max(0.05, sdconv(1, Data@CV_Cat[x]), na.rm = TRUE) %>% log()
+  }
+  if(is.null(params$log_sigma)) {
+    params$log_sigma <- max(0.05, sdconv(1, Data@CV_Ind[x]), na.rm = TRUE) %>% log()
   }
   if(is.null(params$log_tau)) {
-    tau_start <- ifelse(is.na(Data@sigmaR[x]), 0.6, Data@sigmaR[x])
-    params$log_tau <- log(tau_start)
+    params$log_tau <- ifelse(is.na(Data@sigmaR[x]), 0.6, Data@sigmaR[x]) %>% log()
   }
   params$log_rec_dev = rep(0, ny - k)
 
   info <- list(Year = Year, data = data, params = params, I_hist = I_hist, LH = LH,
-               rescale = rescale, E_rescale = E_rescale, control = control, inner.control = inner.control)
+               rescale = rescale, control = control, inner.control = inner.control)
+  if(condition == "effort") info$E_rescale <- E_rescale
 
   map <- list()
+  if(condition == "catch") map$log_q_effort <- factor(NA)
   if(fix_h) map$transformed_h <- factor(NA)
   if(fix_U_equilibrium) map$U_equilibrium <- factor(NA)
   if(fix_omega) map$log_omega <- factor(NA)
+  if(fix_sigma) map$log_sigma <- factor(NA)
   if(fix_tau) map$log_tau <- factor(NA)
+  if(!state_space) map$log_rec_dev <- factor(rep(NA, ny - k))
 
   random <- NULL
   if(integrate) random <- "log_rec_dev"
@@ -324,21 +232,12 @@ DD_SS <- function(x = 1, Data, SR = c("BH", "Ricker"), rescale = "mean1", start
   SD <- mod[[2]]
   report <- obj$report(obj$env$last.par.best)
 
-  if(rescale != 1) {
-    vars_div <- c("B0", "B", "Cpred", "N0", "N", "R0", "R")
-    vars_mult <- c("Brec")
-    var_trans <- c("R0")
-    fun_trans <- c("/")
-    fun_fixed <- c("log")
-    rescale_report(vars_div, vars_mult, var_trans, fun_trans, fun_fixed)
-  }
-
   Yearplusone <- c(Year, max(Year) + 1)
   Yearplusk <- c(Year, max(Year) + 1:k)
-  YearDev <- seq(Year[1] + k, max(Year))
 
   nll_report <- ifelse(is.character(opt), ifelse(integrate, NA, report$nll), opt$objective)
-  Assessment <- new("Assessment", Model = "DD_SS", Name = Data@Name, conv = !is.character(SD) && SD$pdHess,
+  Assessment <- new("Assessment", Model = ifelse(state_space, "DD_SS", "DD_TMB"),
+                    Name = Data@Name, conv = !is.character(SD) && SD$pdHess,
                     B0 = report$B0, R0 = report$R0, N0 = report$N0,
                     SSB0 = report$B0, VB0 = report$B0, h = report$h,
                     U = structure(report$U, names = Year),
@@ -353,35 +252,42 @@ DD_SS <- function(x = 1, Data, SR = c("BH", "Ricker"), rescale = "mean1", start
                     Obs_Catch = structure(C_hist, names = Year),
                     Obs_Index = structure(I_hist, names = Year),
                     Catch = structure(report$Cpred, names = Year),
-                    Index = structure(report$Cpred/(C_hist/I_hist), names = Year),
-                    Dev = structure(report$log_rec_dev, names = YearDev),
-                    Dev_type = "log-Recruitment deviations",
+                    Index = structure(report$Ipred, names = Year),
                     NLL = structure(c(nll_report, report$nll_comp, report$penalty),
-                                    names = c("Total", "Catch", "Dev", "Penalty")),
+                                    names = c("Total", ifelse(condition == "catch", "Index", "Catch"),
+                                              "Dev", "Penalty")),
                     info = info, obj = obj, opt = opt, SD = SD, TMB_report = report,
                     dependencies = dependencies)
 
+  if(state_space) {
+    YearDev <- seq(Year[1] + k, max(Year))
+    Assessment@Dev <- structure(report$log_rec_dev, names = YearDev)
+    Assessment@Dev_type <- "log-Recruitment deviations"
+  }
+
   if(Assessment@conv) {
     ref_pt <- get_MSY_DD(info$data, report$Arec, report$Brec)
     report <- c(report, ref_pt)
 
-    if(integrate) {
-      SE_Dev <- sqrt(SD$diag.cov.random)
-    } else {
-      SE_Dev <- sqrt(diag(SD$cov.fixed)[names(SD$par.fixed) == "log_rec_dev"])
-    }
-
     Assessment@UMSY <- report$UMSY
     Assessment@MSY <- report$MSY
     Assessment@BMSY <- Assessment@SSBMSY <- Assessment@VBMSY <- report$BMSY
     Assessment@U_UMSY <- structure(report$U/report$UMSY, names = Year)
     Assessment@B_BMSY <- Assessment@SSB_SSBMSY <- Assessment@VB_VBMSY <- structure(report$B/report$BMSY, names = Yearplusone)
-    Assessment@SE_Dev <- structure(SE_Dev, names = YearDev)
     Assessment@TMB_report <- report
+
+    if(state_space) {
+      if(integrate) {
+        SE_Dev <- sqrt(SD$diag.cov.random)
+      } else {
+        SE_Dev <- sqrt(diag(SD$cov.fixed)[names(SD$par.fixed) == "log_rec_dev"])
+      }
+      Assessment@SE_Dev <- structure(SE_Dev, names = YearDev)
+    }
+
   }
   return(Assessment)
 }
-class(DD_SS) <- "Assess"
 
 
 get_MSY_DD <- function(TMB_data, Arec, Brec) {
