@@ -12,10 +12,51 @@
 #' @slot CAA An array for the predicted catch at age with dimension \code{OM@@nsim}, \code{OM@@nyears}, \code{OM@@maxage}, and nfleet.
 #' @slot CAL An array for the predicted catch at length with dimension \code{OM@@nsim}, \code{OM@@nyears}, length bins, and nfleet.
 #' @slot conv A logical vector of length \code{OM@@nsim} indicating convergence of the SRA scoping model in the i-th simulation.
-#' @slot Misc A list of length \code{OM@@nsim} with more output from the fitted SRA scoping model.
-#' @slot mean_fit A list of output from fit to mean values of life history parameters in the operating model.
+#' @slot Misc A list of length \code{OM@@nsim} with more output from the fitted SRA scoping model. Within each simulation, items of interest include:
+#'
+#' \itemize{
+#' \item B - total biomass - vector of length nyears+1
+#' \item E0 - annual unfished spawning biomass - vector of length nyears
+#' \item E0_SR - unfished spawning biomass for the stock-recruit relationship - numeric
+#' \item Arec - alpha parameter of the stock-recruit relationship - numeric
+#' \item Brec - beta parameter of the stock-recruit relationship - numeric
+#' \item R - recruitment - vector of length nyears+1
+#' \item R_early - recruitment for the cohorts in first year of the model - vector maxage-1
+#' \item VB - vulnerable biomass - matrix of nyears x nfleet
+#' \item N - abundance at age - matrix of nyears+1 x maxage
+#' \item F - apical fishing mortality - matrix of nyears x nfleet
+#' \item F_at_age - fishing mortality at age - array of nyears x maxage x nfleet
+#' \item F_equilibrium - equilibrium fishing mortality prior to first year - vector of length nfleet
+#' \item M - natural mortality - matrix of nyears x maxage
+#' \item Z - total mortality - matrix of nyears x maxage
+#' \item q - survey catchability - vector of length nsurvey
+#' \item s_vul - survey selectivity at age - array of dim nyears+1, maxage, nsurvey
+#' \item s_vul_len - corresponding survey selectivity at length - matrix of nbins x nsurvey
+#' \item Ipred - predicted index values - matrix of nyears x nsurvey
+#' \item s_CAApred - predicted survey catch at age - array of dim nyears, maxage, nsurvey
+#' \item vul - fleet selectivity at age - array of dim nyears+1, maxage, nfleet (or nsel_block)
+#' \item vul_len - corresponding fleet selectivity at length - matrix of nbins x nfleet (or nsel_block)
+#' \item s_CALpred - predicted survey catch at length - array of dim nyears, nbins, nsurvey
+#' \item mlen_pred - predicted mean lengths - matrix of nyears x nfleet
+#' \item CAApred - predicted catch at age - array of nyears, maxage, nfleet
+#' \item CALpred - predicted catch at length - array of nyears, nbins, nfleet
+#' \item Cpred - predicted catch in weight - matrix of nyears x nfleet
+#' \item CN - predicted catch in numbers - matrix of nyears x nfleet
+#' \item nll - Total objective function of the model - numeric
+#' }
+#'
+#' @slot mean_fit A list of output from fit to mean values of life history parameters in the operating model. The named list consists of:
+#'
+#' \itemize{
+#' \item obj - a list with components returned from \code{\link[TMB]{MakeADFun}}.
+#' \item opt - a list with components from calling \code{\link[stats]{nlminb}} to \code{obj}.
+#' \item SD - a list (class sdreport) with parameter estimates and their standard errors, obtained from
+#' \code{\link[TMB]{sdreport}}.
+#' \item report - a list of model output reported from the TMB executable, i.e. \code{obj$report()}. See Misc.
+#' }
 #' @slot data A list of the data inputs for the SRA scoping model.
-#' @slot config A data frame describing configuration of the SRA scoping model.
+#' @slot config A data frame describing configuration of the SRA scoping model (not currently used).
+#'
 #' @seealso \link{plot.SRA} \link{SRA_scope}
 #' @author Q. Huynh
 #' @export SRA
@@ -229,7 +270,7 @@ setMethod("plot", signature(x = "SRA", y = "missing"),
               # Data and fit section
               individual_matrix_fn <- function(i, obs, pred, fig.cap, label) {
                 rmd_assess_fit2("Year", paste0(obs, "[, ", i, "]"), paste0(pred, "[, ", i, "]"),
-                                fig.cap = paste(fig.cap, i), label = eval(parse(text = paste0(label, "[", i, "]"))))
+                                fig.cap = paste(fig.cap, i), label = label[i])
               }
               individual_array_fn <- function(i, obs, pred, comps = c("age", "length"), label) {
                 comps <- match.arg(comps)
@@ -254,7 +295,7 @@ setMethod("plot", signature(x = "SRA", y = "missing"),
 
                 if(data_mean_fit$condition == "effort" || ncol(data$Chist) > 1) {
                   C_plots <- lapply(1:nfleet, individual_matrix_fn, obs = "data$Chist", pred = "report$Cpred",
-                                    fig.cap = "catch from fleet", label = "f_name")
+                                    fig.cap = "catch from fleet", label = f_name)
                 } else C_plots <- NULL
               } else C_matplot <- C_plots <- NULL
 
@@ -264,14 +305,14 @@ setMethod("plot", signature(x = "SRA", y = "missing"),
                 } else {
                   E_header <- "#### Effort \n"
                 }
-                E_matplot <- rmd_matplot(x = "matrix(Year, nyears, nfleet)", y = "data_mean_fit$Ehist", col = "rich.colors(nfleet)",
+                E_matplot <- rmd_matplot(x = "matrix(Year, nyears, nfleet)", y = "data_mean_fit$E_hist", col = "rich.colors(nfleet)",
                                          xlab = "Year", ylab = "Effort", legend.lab = "f_name", fig.cap = "Effort time series.", header = E_header)
               } else E_matplot <- NULL
 
               if(any(data$Index > 0, na.rm = TRUE)) {
                 I_plots <- c("#### Surveys \n",
                              lapply(1:nsurvey, individual_matrix_fn, obs = "data$Index", pred = "report$Ipred",
-                                    fig.cap = "index from survey", label = "s_name"))
+                                    fig.cap = "index from survey", label = s_name))
               } else I_plots <- NULL
 
               if(any(data$CAA > 0, na.rm = TRUE)) {
@@ -285,9 +326,10 @@ setMethod("plot", signature(x = "SRA", y = "missing"),
               } else CAL_plots <- NULL
 
               if(any(data$ML > 0, na.rm = TRUE)) {
+                ML_label <- paste("Mean Length from", f_name)
                 ML_plots <- c("#### Mean lengths \n",
                               lapply(1:nfleet, individual_matrix_fn, obs = "data$ML", pred = "report$mlen_pred",
-                                     fig.cap = "mean lengths from fleet", label = paste("Mean Length from", f_name)))
+                                     fig.cap = "mean lengths from fleet", label = ML_label))
               } else ML_plots <- NULL
 
               if(any(data$s_CAA > 0, na.rm = TRUE)) {
